# Hunter's Say System Addon
#  
# Install instructions:
#       1. Install Mattie's EventScripts 2.0 plugin:
#           http://mattie.info/cs/
#
#       2. Copy and upload this script to:
#           <gamedir>/addons/eventscripts/say_system/
#
#       3. Add the following line somewhere in autoexec.cfg:
#           es_load say_system
#
################################################################################
#import EventScripts
import es
#import Libraries
import services
import popuplib
import playerlib
import keyvalues
import settinglib
import gamethread
#import Psyco Compiler
import psyco
psyco.full()

# Addon Information
info = es.AddonInfo()
info.name = "Say System"
info.version = "3.0.0"
info.author = "Hunter"
info.url = "http://addons.eventscripts.com/addons/user/289"
info.description = "Play sounds or view a list of sounds"
info.basename = "say_system"

hunter_say_system_ver   = info.version
hunter_say_system_text  = 'Hunters '+info.name+', www.sourceplugins.de, '+info.version+', ES 2.0.0.247+ (Python Version)'

# Server Variables
say_system_savetime     = es.ServerVar('say_system_savetime', '30', 'How much days should inactive users be stored, before their settings get deleted? ( default = 30 )')
say_system_soundload    = es.ServerVar('say_system_soundload', '1', 'Should the sounds be downloaded with EventScripts?')
say_system_default      = es.ServerVar('say_system_default', '1', 'Default settings for new connecting players')
say_system_limit        = es.ServerVar('say_system_limit', '2', 'How much sounds can normal (non-admin) players play each round?')
say_system_force        = es.ServerVar('say_system_force', '0', 'Should the admin sounds always play?')
say_system_show         = es.ServerVar('say_system_show', '1', 'Should the sound trigger be shown?')

# KeyValues Object
say_system_kv           = keyvalues.KeyValues(name='say_system')

# Popuplib Object
say_system_normal       = popuplib.easymenu('say_system_normal', None, None)
say_system_admin        = popuplib.easymenu('say_system_admin', None, None)
say_system_normal_cmd   = popuplib.easymenu('say_system_normal_cmd', None, None)
say_system_admin_cmd    = popuplib.easymenu('say_system_admin_cmd', None, None)

# Settinglib Object
say_system_setting      = settinglib.create('saysounds', 'Say System Setting', 'toggle')

# Global Variables
say_system_players      = {}

def load():
    public = es.ServerVar('hu_so', info.version, info.name)
    public.makepublic()
    
    auth = services.use('auth')
    auth.registerCapability('admin_sounds', auth.ADMIN)
    auth.registerCapability('admin_commands', auth.ADMIN)
    
    say_system_kv.load(es.getAddonPath('say_system')+'/say_system.txt')
    for sound in say_system_kv['sounds_normal'].keys():
        say_system_normal.addoption(sound, sound)
        say_system_admin.addoption(sound, sound)
    for sound in say_system_kv['sounds_admin'].keys():
        say_system_admin.addoption(sound, sound)
    for command in say_system_kv['commands_normal'].keys():
        say_system_normal_cmd.addoption(command, command)
        say_system_admin_cmd.addoption(command, command)
    for command in say_system_kv['commands_admin'].keys():
        say_system_admin_cmd.addoption(command, command)

    say_system_normal.menuselectfb = _select_sound
    say_system_normal.settitle('Say System')
    say_system_normal.vguititle = 'Say System'
    say_system_admin.menuselectfb = _select_sound
    say_system_admin.settitle('Say System Admin')
    say_system_admin.vguititle = 'Say System Admin'

    say_system_normal_cmd.menuselectfb = _select_sound
    say_system_normal_cmd.settitle('Say System')
    say_system_normal_cmd.vguititle = 'Say System'
    say_system_admin_cmd.menuselectfb = _select_sound
    say_system_admin_cmd.settitle('Say System Admin')
    say_system_admin_cmd.vguititle = 'Say System Admin'

    say_system_setting.addoption('normal', 'Player Sounds')
    say_system_setting.addoption('admin', 'Admin Sounds')
    say_system_setting.setdefault('normal', str(say_system_default))
    say_system_setting.setdefault('admin', str(say_system_default))
    say_system_setting.addsound('ui/buttonclick.wav')

    es.regsaycmd('!sounds', 'say_system/saycmd', 'Say System Setting')
    es.regsaycmd('!soundlist', 'say_system/listcmd', 'Say System List')
    es.regsaycmd('!commandlist', 'say_system/listcmd_cmd', 'Say System List')
    es.addons.registerSayFilter(playsound)
    es.log(hunter_say_system_text)
    es.msg('#multi', '#green[SaySystem] #defaultLoaded')
    
def unload():
    es.addons.unregisterSayFilter(playsound)
    es.unregsaycmd('!sounds')
    es.unregsaycmd('!soundlist')
    es.unregsaycmd('!commandlist')
    es.msg('#multi', '#green[SaySystem] #defaultUnloaded')
    
def es_map_start(event_var):
    global say_system_players
    say_system_setting.clear(int(say_system_savetime)*86400)
    say_system_players = {}
    if int(say_system_soundload):
        for key in say_system_kv.keys():
            for sound in say_system_kv[key].keys():
                es.stringtable('downloadables', 'sound/'+say_system_kv[key][sound])

def es_client_command(event_var):
    if (str(event_var['command']) == '!hunter_say_system_ver') or (str(event_var['command']) == '!hunter_all_ver'):
        es.cexec(int(event_var['userid']), 'echo '+hunter_say_system_text)

def player_activate(event_var):
    say_system_setting.updateTime(int(event_var['userid']))
    gamethread.delayed(30, es.cexec, (int(event_var['userid']), 'echo '+hunter_say_system_text))
    gamethread.delayed(15, es.tell, (int(event_var['userid']), '#multi', '#green[SaySystem] #defaultSay \'!soundlist\' to see all sounds'))

def player_disconnect(event_var):
    say_system_setting.updateTime(int(event_var['userid']))
    if int(event_var['userid']) in say_system_players:
        del say_system_players[int(event_var['userid'])]

def playsound(userid, text, teamonly):
    done = False
    auth = services.use('auth')
    if not userid in say_system_players:
        say_system_players[userid] = 0
    for alias in say_system_kv['alias'].keys():
        if text.strip('"').startswith(alias):
            if teamonly:
                gamethread.queue(es.sexec, (userid, 'say_team %s' % say_system_kv['alias'][alias]))
            else:
                gamethread.queue(es.sexec, (userid, 'say %s' % say_system_kv['alias'][alias]))
            return (userid, text, teamonly)
    if auth.isUseridAuthorized(userid, 'admin_sounds'):
        for sound in say_system_kv['sounds_admin'].keys():
            if text.strip('"').startswith(sound):
                done = finalizesound(userid, 'sounds_admin', sound)
        for sound in say_system_kv['sounds_normal'].keys():
            if text.strip('"').startswith(sound):
                done = finalizesound(userid, 'sounds_normal', sound)
    elif say_system_players[userid] < int(say_system_limit):
        for sound in say_system_kv['sounds_normal'].keys():
            if text.strip('"').startswith(sound):
                done = finalizesound(userid, 'sounds_normal', sound)
    if auth.isUseridAuthorized(userid, 'admin_commands'):
        for command in say_system_kv['commands_admin'].keys():
            if text.strip('"').startswith(command):
                done = finalizecommand(userid, 'commands_admin', command, text.strip('"'))
        for command in say_system_kv['commands_normal'].keys():
            if text.strip('"').startswith(command):
                done = finalizecommand(userid, 'commands_normal', command, text.strip('"'))
    elif say_system_players[userid] < int(say_system_limit):
        for command in say_system_kv['commands_normal'].keys():
            if text.strip('"').startswith(command):
                done = finalizecommand(userid, 'commands_normal', command, text.strip('"'))
    if done and not int(say_system_show):
        return (0, None, None)
    else:
        return (userid, text, teamonly)

def finalizesound(userid, key, sound):
    auth = services.use('auth')
    es.msg('#multi', '#green[SaySystem] #lightgreen%s #defaultplayed sound #lightgreen%s' % (es.getplayername(userid), sound))
    if userid in say_system_players:
        say_system_players[userid] += 1
    for target in playerlib.getUseridList('#human'):
        if (key == 'sounds_normal' and say_system_setting.get('normal', target)) or (key == 'sounds_admin' and say_system_setting.get('admin', target)) or (int(say_system_force) and auth.isUseridAuthorized(userid, 'admin_sounds')):
            es.playsound(target, say_system_kv[key][sound], 1.0)
    return True

def finalizecommand(userid, key, command, text):
    args = say_system_kv[key][command]
    argc = args.count('$')+args.count('#')
    if len(text.split(' ')) > argc:
        args = args.replace('%u', str(userid))
        for argv in range(1, argc+1):
            if '$%s'%argv in args: args = args.replace('$%s'%argv, str(text.split(' ')[argv]))
            if '#%s'%argv in args: args = args.replace('#%s'%argv, str(es.getuserid(text.split(' ')[argv])))
            if '#%s'%argv in args and not es.getuserid(text.split(' ')[argv]):
                es.tell(userid, '#multi', '#green[SaySystem] #lightgreenInvalid parameters for command #green%s' % command)
                break
        es.server.cmd(args)
    else:
        es.tell(userid, '#multi', '#green[SaySystem] #lightgreenNot enough parameters for command #green%s' % command)
    return True

def saycmd():
    say_system_setting.send(int(es.getcmduserid()))

def listcmd():
    auth = services.use('auth')
    if auth.isUseridAuthorized(int(es.getcmduserid()), 'admin_sounds'):
        say_system_admin.send(int(es.getcmduserid()))
    else:
        say_system_normal.send(int(es.getcmduserid()))

def listcmd_cmd():
    auth = services.use('auth')
    if auth.isUseridAuthorized(int(es.getcmduserid()), 'admin_commands'):
        say_system_admin_cmd.send(int(es.getcmduserid()))
    else:
        say_system_normal_cmd.send(int(es.getcmduserid()))

def _select_sound(userid, choice, popupid):
    es.sexec(userid, 'say %s' % choice)
